'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SeedlinkConnection = exports.SEEDLINK_PROTOCOL = exports.RSVP = exports.miniseed = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

/**
 * Philip Crotwell
 * University of South Carolina, 2016
 * http://www.seis.sc.edu
 */

/*global DataView*/

var _seisplotjsMiniseed = require('seisplotjs-miniseed');

var miniseed = _interopRequireWildcard(_seisplotjsMiniseed);

var _rsvp = require('rsvp');

var RSVP = _interopRequireWildcard(_rsvp);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* reexport */
exports.miniseed = miniseed;
exports.RSVP = RSVP;
var SEEDLINK_PROTOCOL = exports.SEEDLINK_PROTOCOL = "SeedLink3.1";

RSVP.on('error', function (reason) {
  console.assert(false, reason);
});

var SeedlinkConnection = exports.SeedlinkConnection = function () {
  /** creates a seedlink websocket connection to the given url.
    * requestConfig is an array of seedlink commands
    * like:
    *   [ 'STATION JSC CO',
    *     'SELECT 00BHZ.D' ]
    * and receiveMiniseedFn is the callback function that
    * will be invoked for each seedlink packet received
    * which contains 'sequence', a sequence number
    * and 'miniseed', a single miniseed record.
    * The connection is not made until the connect() method is called.
    */
  function SeedlinkConnection(url, requestConfig, receiveMiniseedFn, errorFn) {
    _classCallCheck(this, SeedlinkConnection);

    this.url = url;
    this.requestConfig = requestConfig;
    this.receiveMiniseedFn = receiveMiniseedFn;
    this.errorFn = errorFn;
    this.command = 'DATA';
  }

  _createClass(SeedlinkConnection, [{
    key: 'setTimeCommand',
    value: function setTimeCommand(startDate) {
      this.command = "TIME " + (0, _moment2.default)(startDate).format("YYYY,MM,DD,HH,mm,ss");
    }
  }, {
    key: 'connect',
    value: function connect() {
      this.webSocket = new WebSocket(this.url, SEEDLINK_PROTOCOL);
      this.webSocket.binaryType = 'arraybuffer';
      var that = this;
      this.webSocket.onopen = function () {
        that.sendHello(that.webSocket).then(function () {
          return that.sendCmdArray(that.webSocket, that.requestConfig);
        }).then(function () {
          return that.sendCmdArray(that.webSocket, [that.command]);
        }).then(function (val) {
          that.webSocket.onmessage = function (event) {
            that.handle(event);
          };
          that.webSocket.send('END\r');
          return val;
        }, function (err) {
          console.assert(false, "reject: " + err);
          that.close();
        });
      };
    }
  }, {
    key: 'close',
    value: function close() {
      if (this.webSocket) {
        this.webSocket.close();
      }
    }
  }, {
    key: 'handle',
    value: function handle(event) {
      //for flow
      var data = event.data;
      if (data.byteLength < 64) {
        //assume text
      } else {
        this.handleMiniseed(event);
      }
    }
  }, {
    key: 'handleMiniseed',
    value: function handleMiniseed(event) {
      //for flow
      var data = event.data;
      try {
        // let arrBuf = new ArrayBuffer(event.data);
        if (data.byteLength < 64) {
          this.errorFn(new Error("message too small to be miniseed: " + data.byteLength + " " + (0, _util.dataViewToString)(new DataView(data))));
          return;
        }
        var slHeader = new DataView(data, 0, 8);
        // check for 'SL' at start
        if (slHeader.getInt8(0) === 83 && slHeader.getInt8(1) === 76) {
          var seqStr = '';
          for (var i = 0; i < 6; i++) {
            seqStr = seqStr + String.fromCharCode(slHeader.getInt8(2 + i));
          }
          var dataView = new DataView(data, 8, data.byteLength - 8);
          var out = {
            rawsequence: seqStr,
            sequence: parseInt(seqStr, 16),
            miniseed: miniseed.parseSingleDataRecord(dataView)
          };
          this.receiveMiniseedFn(out);
        } else {
          this.errorFn(new Error("Not a seedlink packet, no starting SL: " + slHeader.getInt8(0) + ' ' + slHeader.getInt8(1)));
        }
      } catch (e) {
        console.assert(false, e);
        this.errorFn("Error, closing seedlink. " + e);
        this.close();
      }
    }
  }, {
    key: 'sendHello',
    value: function sendHello(webSocket) {
      var promise = new RSVP.Promise(function (resolve, reject) {
        webSocket.onmessage = function (event) {
          //for flow
          var data = event.data;
          var replyMsg = (0, _util.dataViewToString)(new DataView(data));
          var lines = replyMsg.trim().split('\r');
          if (lines.length == 2) {
            resolve(lines);
          } else {
            reject("not 2 lines: " + replyMsg);
          }
        };
        webSocket.send("HELLO\r");
      });
      return promise;
    }
  }, {
    key: 'sendCmdArray',
    value: function sendCmdArray(webSocket, cmd) {
      var that = this;
      return cmd.reduce(function (accum, next) {
        return accum.then(function () {
          return that.createCmdPromise(webSocket, next);
        });
      }, RSVP.resolve());
    }
  }, {
    key: 'createCmdPromise',
    value: function createCmdPromise(webSocket, mycmd) {
      var promise = new RSVP.Promise(function (resolve, reject) {
        webSocket.onmessage = function (event) {
          //for flow
          var data = event.data;
          var replyMsg = (0, _util.dataViewToString)(new DataView(data)).trim();
          if (replyMsg === 'OK') {
            resolve(replyMsg);
          } else {
            reject("msg not OK: " + replyMsg);
          }
        };
        webSocket.send(mycmd + '\r\n');
      });
      return promise;
    }
  }]);

  return SeedlinkConnection;
}();